@@ -157,9 +157,12 @@ class Bin:
         Bin(0b0, n=0)
         >>> Bin.empty(4)
         Bin(0b0000, n=4)
+        >>> Bin.zero(4)
+        Bin(0b0000, n=4)
         """
         n = _check_n(n)
         return self._new(0, n)
+    zero = empty
 
     @classmethod
     def full(self, n):
@@ -169,9 +172,23 @@ class Bin:
         Bin(0b0, n=0)
         >>> Bin.full(4)
         Bin(0b1111, n=4)
+        >>> Bin.ones(4)
+        Bin(0b1111, n=4)
         """
         n = _check_n(n)
         return self._new(2**n-1, n)
+    ones = full
+
+    @classmethod
+    def iter(self, n):
+        """Iterate over all `n`-bit :class:`Bin`s.
+
+        >>> list(Bin.iter(2))
+        [Bin(0b00, n=2), Bin(0b01, n=2), Bin(0b10, n=2), Bin(0b11, n=2)]
+        """
+        n = _check_n(n)
+        for x in range(2**n):
+            yield Bin(x, n)
 
     @classmethod
     def unit(cls, i, n):
@@ -449,13 +466,28 @@ class Bin:
         0
         >>> Bin("101010")[2]
         1
+        >>> Bin("101010")[5]
+        0
+        >>> Bin("101010")[6]
+        Traceback (most recent call last):
+        IndexError: index 6 out of range (-6,5)
+        >>> Bin("101010")[-6]
+        1
+        >>> Bin("101010")[-7]
+        Traceback (most recent call last):
+        IndexError: index -7 out of range (-6,5)
         """
         if isinstance(idx, slice):
             # todo: optimize simple substring slices?
             # easy to mess up with out of bounds, negative indices, etc. ...
             return Bin(self.tuple[idx])
         else:
-            idx = int(idx) % self.n
+            idx = int(idx)
+            if idx < -self.n or idx >= self.n:
+                raise IndexError(f"index {idx} out of range ({-self.n},{self.n-1})")
+            if idx < 0:
+                idx += self.n
+            assert 0 <= idx < self.n
             return 1 & (self.int >> (self.n - 1 - idx))
 
     # __setitem__ maybe ?
@@ -663,16 +695,57 @@ class Bin:
         >>> Bin(31, 5) + Bin(1, 5)
         Bin(0b00000, n=5)
         >>> Bin(31, 5) + Bin(1, 6)
-        Bin(0b100000, n=6)
+        Traceback (most recent call last):
+         ...
+        ValueError: add/sub Bin with different n? 5 vs 6
         """
         if isinstance(other, Bin):
-            n = max(self.n, other.n)
+            if self.n != other.n:
+                raise ValueError(f"add/sub Bin with different n? {self.n} vs {other.n}")
             other = other.int
-        else:
-            n = self.n
-        mod = 1 << n
-        v = int(self.int + other) % mod
-        return self._new(v, n=n)
+        mask = (1 << self.n) - 1
+        v = int(self.int + other) & mask
+        return self._new(v, n=self.n)
+
+    __radd__ = __add__
+
+    def __sub__(self, other):
+        """
+        >>> Bin(1, 5) - 2
+        Bin(0b11111, n=5)
+        >>> Bin(17, 5) - Bin(1, 5)
+        Bin(0b10000, n=5)
+        >>> Bin(1, 5) + Bin(31, 6)
+        Traceback (most recent call last):
+         ...
+        ValueError: add/sub Bin with different n? 5 vs 6
+        """
+        if isinstance(other, Bin):
+            if self.n != other.n:
+                raise ValueError(f"add/sub Bin with different n? {self.n} vs {other.n}")
+            other = other.int
+        mask = (1 << self.n) - 1
+        v = int(self.int - other) & mask
+        return self._new(v, n=self.n)
+
+    def __rsub__(self, other):
+        """
+        >>> Bin(1, 5) - 2
+        Bin(0b11111, n=5)
+        >>> Bin(17, 5) - Bin(1, 5)
+        Bin(0b10000, n=5)
+        >>> Bin(1, 5) + Bin(31, 6)
+        Traceback (most recent call last):
+         ...
+        ValueError: add/sub Bin with different n? 5 vs 6
+        """
+        if isinstance(other, Bin):
+            if self.n != other.n:
+                raise ValueError(f"add/sub Bin with different n? {self.n} vs {other.n}")
+            other = other.int
+        mask = (1 << self.n) - 1
+        v = int(other - self.int) & mask
+        return self._new(v, n=self.n)
 
     def scalar_bin(self, other):
         """Dot product in GF(2).
